@@ -1,10 +1,10 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
 
-#include <QtWidgets>
-
 #include "audiodevices.h"
 
+#include <QApplication>
+
 int main(int argv, char **args)
 {
     QApplication app(argv, args);
