@@ -3,7 +3,7 @@
 
 #include "qaudiosystem_p.h"
 #include "qaudiodevice_p.h"
-#include <private/qplatformmediadevices_p.h>
+#include <private/qplatformaudiodevices_p.h>
 #include <private/qplatformmediaintegration_p.h>
 
 #include <QtCore/qmap.h>
@@ -30,6 +30,11 @@ QT_DEFINE_QESDP_SPECIALIZATION_DTOR(QAudioDevicePrivate);
     QAudioSource, and QAudioSink. It is also used to determine the
     input or output device to use in a capture session or during media playback.
 
+    The QAudioDevice instance retains its properties throughout its lifetime,
+    even if the corresponding physical device is disconnected or its settings are
+    modified. To keep track of updated properties, the user should load new instances
+    of QAudioDevice from \l{QMediaDevices} when the relevant signals are fired.
+
     You can also query each device for the formats it supports. A
     format in this context is a set consisting of a channel count, sample rate, and sample type. A
     format is represented by the QAudioFormat class.
@@ -60,7 +65,7 @@ QT_DEFINE_QESDP_SPECIALIZATION_DTOR(QAudioDevicePrivate);
     \qmlvaluetype audioDevice
     \inqmlmodule QtMultimedia
     \since 6.2
-    //! \instantiates QAudioDevice
+    //! \nativetype QAudioDevice
     \brief Describes an audio device.
     \ingroup multimedia_qml
     \ingroup multimedia_audio_qml
@@ -69,6 +74,11 @@ QT_DEFINE_QESDP_SPECIALIZATION_DTOR(QAudioDevicePrivate);
     The audioDevice value type describes the properties of an audio device that
     is connected to the system.
 
+    The audioDevice instance retains its properties throughout its lifetime,
+    even if the corresponding physical device is disconnected or its settings are
+    modified. To keep track of updated properties, the user should load new instances
+    of audioDevice from \l{MediaDevices} when the relevant signals are fired.
+
     The list of audio input or output devices can be queried from the \l{MediaDevices}
     type. To select a certain audio device for input or output set it as the device
     on \l{AudioInput} or \l{AudioOutput}.
@@ -127,13 +137,7 @@ QAudioDevice &QAudioDevice::operator=(const QAudioDevice &other) = default;
 */
 bool QAudioDevice::operator==(const QAudioDevice &other) const
 {
-    if (d == other.d)
-        return true;
-    if (!d || !other.d)
-        return false;
-    if (d->mode == other.d->mode && d->id == other.d->id && d->isDefault == other.d->isDefault)
-        return true;
-    return false;
+    return mode() == other.mode() && id() == other.id();
 }
 
 /*!
@@ -237,7 +241,7 @@ bool QAudioDevice::isFormatSupported(const QAudioFormat &settings) const
 
     These settings are provided by the platform/audio plugin being used.
 
-    They are also dependent on the \l {QAudio}::Mode being used.
+    They are also dependent on the \l {QtAudio}::Mode being used.
 
     A typical audio system would provide something like:
     \list
