@@ -144,6 +144,12 @@ void QAudioSource::start(QIODevice* device)
 {
     if (!d)
         return;
+    if (!device->isWritable()) {
+        qWarning() << "QAudioSource::start: QIODevice is not writable";
+        d->setError(QAudio::OpenError);
+        return;
+    }
+
     d->elapsedTime.start();
     d->start(device);
 }
@@ -329,8 +335,6 @@ qint64 QAudioSource::processedUSecs() const
     Suspend states.
 */
 
-#include <qdebug.h>
-
 qint64 QAudioSource::elapsedUSecs() const
 {
     return state() == QAudio::StoppedState ? 0 : d->elapsedTime.nsecsElapsed()/1000;
