@@ -199,6 +199,11 @@ bool QCameraFormat::operator==(const QCameraFormat &other) const
     availableCameras() and defaultCamera() functions. These are contained within
     QtMultimedia::MediaDevices.
 
+    The QCameraDevice instance retains its properties throughout its lifetime,
+    even if the corresponding physical device is disconnected or its settings are
+    modified. To keep track of updated properties, the user should load new instances
+    of QCameraDevice from \l{QMediaDevices} when the relevant signals are fired.
+
     This example prints the name of all available cameras:
 
     \snippet multimedia-snippets/camera.cpp Camera listing
@@ -229,6 +234,11 @@ bool QCameraFormat::operator==(const QCameraFormat &other) const
     The cameraDevice value type describes the properties of a camera device that
     is connected to the system.
 
+    The cameraDevice instance retains its properties throughout its lifetime,
+    even if the corresponding physical device is disconnected or its settings are
+    modified. To keep track of updated properties, the user should load new instances
+    of cameraDevice from \l{MediaDevices} when the relevant signals are fired.
+
     The list of camera devices can be queried from the \l{MediaDevices}
     type. To select a certain camera device set it as the device
     on \l{Camera}.
