@@ -18,10 +18,13 @@
 #include <QtCore/qurl.h>
 #include <QtCore/qsize.h>
 #include <QtCore/qmimetype.h>
+#include <QtCore/qpointer.h>
+#include <QtCore/qiodevice.h>
 
 #include <QtMultimedia/qmediarecorder.h>
 #include <QtMultimedia/qmediametadata.h>
 #include <QtMultimedia/qmediaformat.h>
+#include <QtMultimedia/private/qerrorinfo_p.h>
 #include <QtCore/private/qglobal_p.h>
 
 QT_BEGIN_NAMESPACE
@@ -55,6 +58,7 @@ public:
     QMediaFormat::VideoCodec videoCodec() const { return mediaFormat().videoCodec(); }
     QMediaFormat::AudioCodec audioCodec() const { return mediaFormat().audioCodec(); }
     QMimeType mimeType() const { return mediaFormat().mimeType(); }
+    QString preferredSuffix() const { return mimeType().preferredSuffix(); }
 
     QMediaRecorder::EncodingMode encodingMode() const { return m_encodingMode; }
     void setEncodingMode(QMediaRecorder::EncodingMode mode) { m_encodingMode = mode; }
@@ -115,14 +119,19 @@ public:
     virtual void setMetaData(const QMediaMetaData &) {}
     virtual QMediaMetaData metaData() const { return {}; }
 
-    QMediaRecorder::Error error() const { return m_error;}
-    QString errorString() const { return m_errorString; }
+    QMediaRecorder::Error error() const { return m_error.code(); }
+    QString errorString() const { return m_error.description(); }
 
     QUrl outputLocation() const { return m_outputLocation; }
     virtual void setOutputLocation(const QUrl &location) { m_outputLocation = location; }
     QUrl actualLocation() const { return m_actualLocation; }
     void clearActualLocation() { m_actualLocation.clear(); }
-    void clearError() { error(QMediaRecorder::NoError, QString()); }
+    void clearError() { updateError(QMediaRecorder::NoError, QString()); }
+
+    QIODevice *outputDevice() const { return m_outputDevice; }
+    void setOutputDevice(QIODevice *device) { m_outputDevice = device; }
+
+    virtual void updateAutoStop() { }
 
 protected:
     explicit QPlatformMediaRecorder(QMediaRecorder *parent);
@@ -130,17 +139,19 @@ protected:
     void stateChanged(QMediaRecorder::RecorderState state);
     void durationChanged(qint64 position);
     void actualLocationChanged(const QUrl &location);
-    void error(QMediaRecorder::Error error, const QString &errorString);
+    void updateError(QMediaRecorder::Error error, const QString &errorString);
     void metaDataChanged();
 
     QMediaRecorder *mediaRecorder() { return q; }
 
+    QString findActualLocation(const QMediaEncoderSettings &settings) const;
+
 private:
     QMediaRecorder *q = nullptr;
-    QMediaRecorder::Error m_error = QMediaRecorder::NoError;
-    QString m_errorString;
+    QErrorInfo<QMediaRecorder::Error> m_error;
     QUrl m_actualLocation;
     QUrl m_outputLocation;
+    QPointer<QIODevice> m_outputDevice;
     qint64 m_duration = 0;
 
     QMediaRecorder::RecorderState m_state = QMediaRecorder::StoppedState;
