@@ -51,7 +51,7 @@ public:
     qint64 processedUSecs() const override;
     QAudio::Error error() const override;
     QAudio::State state() const override;
-    void setFormat(const QAudioFormat &format) override;
+    void setFormat(const QAudioFormat &format);
     QAudioFormat format() const override;
     void setVolume(qreal volume) override;
     qreal volume() const override;
