@@ -17,11 +17,12 @@
 
 #include <qaudioformat.h>
 #include <QtCore/qt_windows.h>
-#include <private/qwindowsiupointer_p.h>
-#include <mfapi.h>
-#include <mmsystem.h>
+#include <QtCore/private/qcomptr_p.h>
 #include <mmreg.h>
 
+#include <optional>
+
+struct IAudioClient;
 struct IMFMediaType;
 
 QT_BEGIN_NAMESPACE
@@ -31,10 +32,14 @@ class QWindowsMediaFoundation;
 namespace QWindowsAudioUtils
 {
     bool formatToWaveFormatExtensible(const QAudioFormat &format, WAVEFORMATEXTENSIBLE &wfx);
+    std::optional<WAVEFORMATEXTENSIBLE> toWaveFormatExtensible(const QAudioFormat &format);
+
     QAudioFormat waveFormatExToFormat(const WAVEFORMATEX &in);
     Q_MULTIMEDIA_EXPORT QAudioFormat mediaTypeToFormat(IMFMediaType *mediaType);
-    QWindowsIUPointer<IMFMediaType> formatToMediaType(QWindowsMediaFoundation &, const QAudioFormat &format);
+    ComPtr<IMFMediaType> formatToMediaType(QWindowsMediaFoundation &, const QAudioFormat &format);
     QAudioFormat::ChannelConfig maskToChannelConfig(UINT32 mask, int count);
+    std::optional<quint32> usedFrames(IAudioClient *client);
+    std::optional<quint32> allocatedFrames(IAudioClient *client);
 }
 
 QT_END_NAMESPACE
