@@ -14,9 +14,9 @@
 // We mean it.
 //
 
-#include "qffmpeghwaccel_p.h"
-#include <private/quniquehandle_p.h>
-#include <private/qcomptr_p.h>
+#include <QtFFmpegMediaPluginImpl/private/qffmpeghwaccel_p.h>
+#include <QtCore/private/quniquehandle_p.h>
+#include <QtCore/private/qcomptr_p.h>
 #include <qt_windows.h>
 
 #include <d3d11.h>
@@ -33,8 +33,8 @@ namespace QFFmpeg {
 struct SharedTextureHandleTraits
 {
     using Type = HANDLE;
-    static Type invalidValue() { return nullptr; }
-    static bool close(Type handle) { return CloseHandle(handle) != 0; }
+    static Type invalidValue() noexcept { return nullptr; }
+    static bool close(Type handle) noexcept { return CloseHandle(handle) != 0; }
 };
 
 using SharedTextureHandle = QUniqueHandle<SharedTextureHandleTraits>;
@@ -56,7 +56,8 @@ public:
     bool copyToSharedTex(ID3D11Device *dev, ID3D11DeviceContext *ctx,
                          const ComPtr<ID3D11Texture2D> &tex, UINT index, const QSize &frameSize);
 
-    /** Obtain a copy of the texture on a second device 'dev' */
+    /** Obtain a copy of the texture on a second device 'dev'.
+     * NOTE: Will block until a texture is available (copyToSharedTex was called) */
     ComPtr<ID3D11Texture2D> copyFromSharedTex(const ComPtr<ID3D11Device1> &dev,
                                               const ComPtr<ID3D11DeviceContext> &ctx);
 
