@@ -17,6 +17,8 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 // Probably, might be increased. To be investigated and tested on Android implementation
 static constexpr int MaxPendingImagesCount = 1;
 
@@ -248,7 +250,7 @@ void QFFmpegImageCapture::setImageSettings(const QImageEncoderSettings &settings
     auto s = settings;
     const auto supportedFormats = QPlatformMediaIntegration::instance()->formatInfo()->imageFormats;
     if (supportedFormats.isEmpty()) {
-        emit error(-1, QImageCapture::FormatError, "No image formats supported, can't capture.");
+        emit error(-1, QImageCapture::FormatError, u"No image formats supported, can't capture."_s);
         return;
     }
     if (s.format() == QImageCapture::UnspecifiedFormat) {
@@ -257,7 +259,7 @@ void QFFmpegImageCapture::setImageSettings(const QImageEncoderSettings &settings
             f = supportedFormats.first();
         s.setFormat(f);
     } else if (!supportedFormats.contains(settings.format())) {
-        emit error(-1, QImageCapture::FormatError, "Image format not supported.");
+        emit error(-1, QImageCapture::FormatError, u"Image format not supported."_s);
         return;
     }
 
