@@ -353,12 +353,12 @@ void QGstreamerImageCapture::convertBufferToImage(const QMutexLocker<QRecursiveM
 void QGstreamerImageCapture::setResolution(const QSize &resolution)
 {
     QGstCaps padCaps = bin.staticPad("sink").currentCaps();
-    if (padCaps.isNull()) {
+    if (!padCaps) {
         qDebug() << "Camera not ready";
         return;
     }
     QGstCaps caps = padCaps.copy();
-    if (caps.isNull())
+    if (!caps)
         return;
 
     gst_caps_set_simple(caps.caps(), "width", G_TYPE_INT, resolution.width(), "height", G_TYPE_INT,
