@@ -8,8 +8,9 @@
 #include <qmediadevices.h>
 #include <qaudiodevice.h>
 #include <private/qmemoryvideobuffer_p.h>
+#include <private/qvideoframe_p.h>
 #include <private/qwindowsmfdefs_p.h>
-#include <private/qwindowsiupointer_p.h>
+#include <QtCore/private/qcomptr_p.h>
 #include <QtCore/qdebug.h>
 
 #include <mmdeviceapi.h>
@@ -22,7 +23,7 @@ QWindowsMediaDeviceReader::QWindowsMediaDeviceReader(QObject *parent)
     : QObject(parent)
 {
     m_durationTimer.setInterval(100);
-    connect(&m_durationTimer, SIGNAL(timeout()), this, SLOT(updateDuration()));
+    connect(&m_durationTimer, &QTimer::timeout, this, &QWindowsMediaDeviceReader::updateDuration);
 }
 
 QWindowsMediaDeviceReader::~QWindowsMediaDeviceReader()
@@ -638,9 +639,9 @@ QMediaRecorder::Error QWindowsMediaDeviceReader::startRecording(
     if (!m_active || m_recording || (videoFormat == GUID_NULL && audioFormat == GUID_NULL))
         return QMediaRecorder::ResourceError;
 
-    QWindowsIUPointer<IMFAttributes> writerAttributes;
+    ComPtr<IMFAttributes> writerAttributes;
 
-    HRESULT hr = MFCreateAttributes(writerAttributes.address(), 2);
+    HRESULT hr = MFCreateAttributes(writerAttributes.GetAddressOf(), 2);
     if (FAILED(hr))
         return QMediaRecorder::ResourceError;
 
@@ -654,9 +655,9 @@ QMediaRecorder::Error QWindowsMediaDeviceReader::startRecording(
     if (FAILED(hr))
         return QMediaRecorder::ResourceError;
 
-    QWindowsIUPointer<IMFSinkWriter> sinkWriter;
+    ComPtr<IMFSinkWriter> sinkWriter;
     hr = MFCreateSinkWriterFromURL(reinterpret_cast<LPCWSTR>(fileName.utf16()),
-                                   nullptr, writerAttributes.get(), sinkWriter.address());
+                                   nullptr, writerAttributes.Get(), sinkWriter.GetAddressOf());
     if (FAILED(hr))
         return QMediaRecorder::LocationNotWritable;
 
@@ -702,7 +703,7 @@ QMediaRecorder::Error QWindowsMediaDeviceReader::startRecording(
     if (FAILED(hr))
         return QMediaRecorder::ResourceError;
 
-    m_sinkWriter = sinkWriter.release();
+    m_sinkWriter = sinkWriter.Detach();
     m_lastDuration = -1;
     m_currentDuration = 0;
     updateDuration();
@@ -955,9 +956,11 @@ STDMETHODIMP QWindowsMediaDeviceReader::OnReadSample(HRESULT hrStatus, DWORD dwS
 
                     if (SUCCEEDED(mediaBuffer->Lock(&buffer, nullptr, &bufLen))) {
                         auto bytes = QByteArray(reinterpret_cast<char*>(buffer), bufLen);
+                        QVideoFrameFormat format(QSize(m_frameWidth, m_frameHeight), m_pixelFormat);
 
-                        QVideoFrame frame(new QMemoryVideoBuffer(bytes, m_stride),
-                                          QVideoFrameFormat(QSize(m_frameWidth, m_frameHeight), m_pixelFormat));
+                        QVideoFrame frame = QVideoFramePrivate::createFrame(
+                                std::make_unique<QMemoryVideoBuffer>(std::move(bytes), m_stride),
+                                std::move(format));
 
                         // WMF uses 100-nanosecond units, Qt uses microseconds
                         frame.setStartTime(llTimestamp * 0.1);
@@ -1012,3 +1015,5 @@ STDMETHODIMP QWindowsMediaDeviceReader::OnMarker(DWORD, LPVOID)
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qwindowsmediadevicereader_p.cpp"
