@@ -1,7 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
-
-//TESTED_COMPONENT=src/multimedia
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtTest/QtTest>
 #include <QtGui/QImageReader>
@@ -13,6 +11,7 @@
 
 #include <private/qplatformcamera_p.h>
 #include <private/qplatformimagecapture_p.h>
+#include <private/qplatformmediaintegration_p.h>
 #include <qcamera.h>
 #include <qcameradevice.h>
 #include <qimagecapture.h>
@@ -27,6 +26,8 @@
 #include <QtCore/private/qcore_mac_p.h>
 #endif
 
+#include <private/mediabackendutils_p.h>
+
 QT_USE_NAMESPACE
 
 /*
@@ -54,6 +55,8 @@ private slots:
     void testCameraFormat();
     void testCameraCapture();
     void testCaptureToBuffer();
+    void captureToFile_createsFileWithExpectedExtension_data();
+    void captureToFile_createsFileWithExpectedExtension();
     void testCameraCaptureMetadata();
     void testExposureCompensation();
     void testExposureMode();
@@ -100,9 +103,9 @@ public Q_SLOTS:
             formatMismatch = 0;
 #ifdef Q_OS_ANDROID
         } else if ((surfaceFormat.pixelFormat() == QVideoFrameFormat::Format_YUV420P
-                   || surfaceFormat.pixelFormat() == QVideoFrameFormat::Format_NV12)
-                  && (cameraFormat.pixelFormat() == QVideoFrameFormat::Format_YUV420P
-                   || cameraFormat.pixelFormat() == QVideoFrameFormat::Format_NV12)
+                   || surfaceFormat.pixelFormat() == QVideoFrameFormat::Format_NV12
+                   || surfaceFormat.pixelFormat() == QVideoFrameFormat::Format_NV21)
+            && cameraFormat.pixelFormat() == QVideoFrameFormat::Format_YUV420P
             && surfaceFormat.frameSize() == cameraFormat.resolution()) {
             formatMismatch = 0;
 #endif
@@ -114,9 +117,6 @@ public Q_SLOTS:
 
 void tst_QCameraBackend::initTestCase()
 {
-#ifdef Q_OS_ANDROID
-    QSKIP("SKIP initTestCase on CI, because of QTBUG-118571");
-#endif
     QCamera camera;
     noCamera = !camera.isAvailable();
 }
@@ -215,8 +215,8 @@ void tst_QCameraBackend::testCameraActive()
     session.setCamera(&camera);
     session.setImageCapture(&imageCapture);
 
-    QSignalSpy errorSignal(&camera, SIGNAL(errorOccurred(QCamera::Error, const QString &)));
-    QSignalSpy activeChangedSignal(&camera, SIGNAL(activeChanged(bool)));
+    QSignalSpy errorSignal(&camera, &QCamera::errorOccurred);
+    QSignalSpy activeChangedSignal(&camera, &QCamera::activeChanged);
 
     QCOMPARE(camera.isActive(), false);
 
@@ -242,6 +242,10 @@ void tst_QCameraBackend::testCameraStartParallel()
 #ifdef Q_OS_ANDROID
     QSKIP("Multi-camera feature is currently not supported on Android. "
           "Cannot open same device twice.");
+#endif
+#ifdef Q_OS_LINUX
+    QSKIP("Multi-camera feature is currently not supported on Linux. "
+          "Cannot open same device twice.");
 #endif
     if (noCamera)
         QSKIP("No camera available");
@@ -275,7 +279,7 @@ void tst_QCameraBackend::testCameraFormat()
     if (videoFormats.isEmpty())
         QSKIP("No Camera available, skipping test.");
     QCameraFormat cameraFormat = videoFormats.first();
-    QSignalSpy spy(&camera, SIGNAL(cameraFormatChanged()));
+    QSignalSpy spy(&camera, &QCamera::cameraFormatChanged);
     QVERIFY(spy.size() == 0);
 
     QMediaCaptureSession session;
@@ -339,9 +343,9 @@ void tst_QCameraBackend::testCameraCapture()
 
     QVERIFY(!imageCapture.isReadyForCapture());
 
-    QSignalSpy capturedSignal(&imageCapture, SIGNAL(imageCaptured(int,QImage)));
-    QSignalSpy savedSignal(&imageCapture, SIGNAL(imageSaved(int,QString)));
-    QSignalSpy errorSignal(&imageCapture, SIGNAL(errorOccurred(int,QImageCapture::Error,const QString&)));
+    QSignalSpy capturedSignal(&imageCapture, &QImageCapture::imageCaptured);
+    QSignalSpy savedSignal(&imageCapture, &QImageCapture::imageSaved);
+    QSignalSpy errorSignal(&imageCapture, &QImageCapture::errorOccurred);
 
     imageCapture.captureToFile();
     QTRY_COMPARE(errorSignal.size(), 1);
@@ -362,7 +366,7 @@ void tst_QCameraBackend::testCameraCapture()
 
     int id = imageCapture.captureToFile();
 
-    QTRY_VERIFY(!savedSignal.isEmpty());
+    QTRY_VERIFY_WITH_TIMEOUT(!savedSignal.isEmpty(), 8s);
 
     QTRY_COMPARE(capturedSignal.size(), 1);
     QCOMPARE(capturedSignal.last().first().toInt(), id);
@@ -398,10 +402,10 @@ void tst_QCameraBackend::testCaptureToBuffer()
 
     QTRY_VERIFY(camera.isActive());
 
-    QSignalSpy capturedSignal(&imageCapture, SIGNAL(imageCaptured(int,QImage)));
-    QSignalSpy imageAvailableSignal(&imageCapture, SIGNAL(imageAvailable(int,QVideoFrame)));
-    QSignalSpy savedSignal(&imageCapture, SIGNAL(imageSaved(int,QString)));
-    QSignalSpy errorSignal(&imageCapture, SIGNAL(errorOccurred(int,QImageCapture::Error,const QString&)));
+    QSignalSpy capturedSignal(&imageCapture, &QImageCapture::imageCaptured);
+    QSignalSpy imageAvailableSignal(&imageCapture, &QImageCapture::imageAvailable);
+    QSignalSpy savedSignal(&imageCapture, &QImageCapture::imageSaved);
+    QSignalSpy errorSignal(&imageCapture, &QImageCapture::errorOccurred);
 
     camera.start();
     QTRY_VERIFY(imageCapture.isReadyForCapture());
@@ -430,6 +434,89 @@ void tst_QCameraBackend::testCaptureToBuffer()
     QTRY_VERIFY(imageCapture.isReadyForCapture());
 }
 
+void tst_QCameraBackend::captureToFile_createsFileWithExpectedExtension_data()
+{
+    QTest::addColumn<QString>("inputFilename");
+    QTest::addColumn<QImageCapture::FileFormat>("format");
+    QTest::addColumn<QString>("expectedFilename");
+
+    {
+        QTest::addRow("Add file extension to JPEG file without extension")
+                << "file" << QImageCapture::JPEG << "file.jpg";
+        QTest::addRow("Keep extension of JPEG file with extension")
+                << "file.jpeg" << QImageCapture::JPEG << "file.jpeg";
+        QTest::addRow("Keep extension of JPEG file with wrong extension")
+                << "file.png" << QImageCapture::JPEG << "file.png";
+    }
+
+    {
+        QTest::addRow("Add file extension to PNG file without extension")
+                << "file" << QImageCapture::PNG << "file.png";
+        QTest::addRow("Keep extension of PNG file with extension")
+                << "file.apng" << QImageCapture::PNG << "file.apng";
+        QTest::addRow("Keep extension of PNG file with wrong extension")
+                << "file.jpg" << QImageCapture::PNG << "file.jpg";
+    }
+
+    {
+        QTest::addRow("Add file extension to WebP file without extension")
+                << "file" << QImageCapture::WebP << "file.webp";
+        QTest::addRow("Keep extension of WebP file with extension")
+                << "file.web" << QImageCapture::WebP << "file.web";
+        QTest::addRow("Keep extension of WebP file with wrong extension")
+                << "file.png" << QImageCapture::WebP << "file.png";
+    }
+
+    {
+        QTest::addRow("Add file extension to Tiff file without extension")
+                << "file" << QImageCapture::Tiff << "file.tiff";
+        QTest::addRow("Keep extension of Tiff file with extension")
+                << "file.cr2" << QImageCapture::Tiff << "file.cr2";
+        QTest::addRow("Keep extension of Tiff file with wrong extension")
+                << "file.png" << QImageCapture::Tiff << "file.png";
+    }
+}
+
+void tst_QCameraBackend::captureToFile_createsFileWithExpectedExtension()
+{
+    if (noCamera)
+        QSKIP("No camera available");
+
+    QFETCH(const QString, inputFilename);
+    QFETCH(const QImageCapture::FileFormat, format);
+    QFETCH(const QString, expectedFilename);
+
+    QCamera camera;
+    camera.setFlashMode(QCamera::FlashOff);
+
+    QImageCapture imageCapture;
+    imageCapture.setFileFormat(format);
+
+    QMediaCaptureSession session;
+    session.setCamera(&camera);
+    session.setImageCapture(&imageCapture);
+
+    QSignalSpy savedSignal(&imageCapture, &QImageCapture::imageSaved);
+
+    camera.start();
+
+    QTRY_VERIFY(imageCapture.isReadyForCapture());
+
+    QTemporaryDir tempDir;
+    tempDir.setAutoRemove(false);
+    const QString tempFile = tempDir.filePath(inputFilename);
+    imageCapture.captureToFile(tempFile);
+
+    QTRY_VERIFY(!savedSignal.isEmpty());
+
+    const QString imagePath = tempDir.filePath(expectedFilename);
+    QVERIFY(QFile::exists(imagePath));
+
+    QImage image;
+    QVERIFY(image.load(imagePath));
+    QVERIFY(!image.isNull());
+}
+
 void tst_QCameraBackend::testCameraCaptureMetadata()
 {
     if (noCamera)
@@ -443,8 +530,14 @@ void tst_QCameraBackend::testCameraCaptureMetadata()
 
     camera.setFlashMode(QCamera::FlashOff);
 
-    QSignalSpy metadataSignal(&imageCapture, SIGNAL(imageMetadataAvailable(int,const QMediaMetaData&)));
-    QSignalSpy savedSignal(&imageCapture, SIGNAL(imageSaved(int,QString)));
+    QMediaMetaData referenceMetaData;
+    referenceMetaData.insert(QMediaMetaData::Title, QStringLiteral("Title"));
+    referenceMetaData.insert(QMediaMetaData::Language, QVariant::fromValue(QLocale::German));
+    referenceMetaData.insert(QMediaMetaData::Description, QStringLiteral("Description"));
+    imageCapture.setMetaData(referenceMetaData);
+
+    QSignalSpy metadataSignal(&imageCapture, &QImageCapture::imageMetadataAvailable);
+    QSignalSpy savedSignal(&imageCapture, &QImageCapture::imageSaved);
 
     camera.start();
 
@@ -455,7 +548,19 @@ void tst_QCameraBackend::testCameraCaptureMetadata()
     int id = imageCapture.captureToFile(tmpFile);
     QTRY_VERIFY(!savedSignal.isEmpty());
     QVERIFY(!metadataSignal.isEmpty());
+
     QCOMPARE(metadataSignal.first().first().toInt(), id);
+    QMediaMetaData receivedMetaData = metadataSignal.first()[1].value<QMediaMetaData>();
+
+    if (isGStreamerPlatform()) {
+        for (auto key : {
+                     QMediaMetaData::Title,
+                     QMediaMetaData::Language,
+                     QMediaMetaData::Description,
+             })
+            QCOMPARE(receivedMetaData[key], referenceMetaData[key]);
+        QVERIFY(receivedMetaData[QMediaMetaData::Resolution].isValid());
+    }
 }
 
 void tst_QCameraBackend::testExposureCompensation()
@@ -467,9 +572,9 @@ void tst_QCameraBackend::testExposureCompensation()
     QCamera camera;
     session.setCamera(&camera);
 
-    QSignalSpy exposureCompensationSignal(&camera, SIGNAL(exposureCompensationChanged(float)));
+    QSignalSpy exposureCompensationSignal(&camera, &QCamera::exposureCompensationChanged);
 
-    //it should be possible to set exposure parameters in Unloaded state
+    // it should be possible to set exposure parameters in Unloaded state
     QCOMPARE(camera.exposureCompensation(), 0.);
     if (!(camera.supportedFeatures() & QCamera::Feature::ExposureCompensation))
         return;
@@ -545,9 +650,10 @@ void tst_QCameraBackend::testVideoRecording_data()
     QTest::addColumn<QCameraDevice>("device");
 
     const auto devices = QMediaDevices::videoInputs();
+    int i = 0;
 
     for (const auto &device : devices)
-        QTest::newRow(device.description().toUtf8()) << device;
+        QTest::addRow("%d - %s", i++, device.description().toUtf8().constData()) << device;
 
     if (devices.isEmpty())
         QTest::newRow("Null device") << QCameraDevice();
@@ -566,16 +672,16 @@ void tst_QCameraBackend::testVideoRecording()
     QMediaRecorder recorder;
     session.setRecorder(&recorder);
 
-    QSignalSpy errorSignal(camera.data(), SIGNAL(errorOccurred(QCamera::Error, const QString &)));
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(QMediaRecorder::Error, const QString &)));
-    QSignalSpy recorderStateChanged(&recorder, SIGNAL(recorderStateChanged(RecorderState)));
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
+    QSignalSpy errorSignal(camera.data(), &QCamera::errorOccurred);
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
+    QSignalSpy recorderStateChanged(&recorder, &QMediaRecorder::recorderStateChanged);
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
 
     recorder.setVideoResolution(320, 240);
 
     // Insert metadata
     QMediaMetaData metaData;
-    metaData.insert(QMediaMetaData::Author, QString::fromUtf8("Author"));
+    metaData.insert(QMediaMetaData::Author, QStringLiteral("Author"));
     metaData.insert(QMediaMetaData::Date, QDateTime::currentDateTime());
     recorder.setMetaData(metaData);
 
@@ -589,8 +695,9 @@ void tst_QCameraBackend::testVideoRecording()
     QTRY_VERIFY(camera->isActive());
 
     recorder.record();
-    durationChanged.clear();
     if (!recorderErrorSignal.empty() || recorderErrorSignal.wait(550)) {
+        QEXPECT_FAIL_GSTREAMER("", "QTBUG-124148: GStreamer might return ResourceError", Continue);
+
         QCOMPARE(recorderErrorSignal.last().first().toInt(), QMediaRecorder::FormatError);
         return;
     }
@@ -614,10 +721,10 @@ void tst_QCameraBackend::testVideoRecording()
     QMediaPlayer player;
     player.setSource(fileName);
 
-    QTRY_COMPARE(player.mediaStatus(), QMediaPlayer::LoadedMedia);
+    QTRY_COMPARE_WITH_TIMEOUT(player.mediaStatus(), QMediaPlayer::LoadedMedia, 8s);
     QCOMPARE_EQ(player.metaData().value(QMediaMetaData::Resolution).toSize(), QSize(320, 240));
     QCOMPARE_GT(player.duration(), 350);
-    QCOMPARE_LT(player.duration(), 550);
+    QCOMPARE_LT(player.duration(), 650);
 
     // TODO: integrate with a virtual camera and check mediaplayer output
 
@@ -637,10 +744,10 @@ void tst_QCameraBackend::testNativeMetadata()
     QMediaRecorder recorder;
     session.setRecorder(&recorder);
 
-    QSignalSpy errorSignal(&camera, SIGNAL(errorOccurred(QCamera::Error, const QString &)));
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
-    QSignalSpy recorderStateChanged(&recorder, SIGNAL(recorderStateChanged(RecorderState)));
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
+    QSignalSpy errorSignal(&camera, &QCamera::errorOccurred);
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
+    QSignalSpy recorderStateChanged(&recorder, &QMediaRecorder::recorderStateChanged);
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
 
     camera.start();
     if (device.isNull()) {
@@ -653,15 +760,15 @@ void tst_QCameraBackend::testNativeMetadata()
     // Insert common metadata supported on all platforms
     // Don't use Date, as some backends set that on their own
     QMediaMetaData metaData;
-    metaData.insert(QMediaMetaData::Title, QString::fromUtf8("Title"));
+    metaData.insert(QMediaMetaData::Title, QStringLiteral("Title"));
     metaData.insert(QMediaMetaData::Language, QVariant::fromValue(QLocale::German));
-    metaData.insert(QMediaMetaData::Description, QString::fromUtf8("Description"));
+    metaData.insert(QMediaMetaData::Description, QStringLiteral("Description"));
 
     recorder.setMetaData(metaData);
 
     recorder.record();
-    durationChanged.clear();
     QTRY_VERIFY(durationChanged.size());
+    QTRY_VERIFY(recorder.recorderState() == QMediaRecorder::RecorderState::RecordingState);
 
     QCOMPARE(recorder.metaData(), metaData);
 
@@ -669,9 +776,13 @@ void tst_QCameraBackend::testNativeMetadata()
     recorder.stop();
 
     QTRY_VERIFY(recorderStateChanged.size() > 0);
+    QTRY_VERIFY(recorder.recorderState() == QMediaRecorder::RecorderState::StoppedState);
 
     QVERIFY(errorSignal.isEmpty());
-    QVERIFY(recorderErrorSignal.isEmpty());
+    if (!isGStreamerPlatform()) {
+        // https://bugreports.qt.io/browse/QTBUG-124183
+        QVERIFY(recorderErrorSignal.isEmpty());
+    }
 
     QString fileName = recorder.actualLocation().toLocalFile();
     QVERIFY(!fileName.isEmpty());
@@ -683,20 +794,25 @@ void tst_QCameraBackend::testNativeMetadata()
     QMediaPlayer player;
     player.setAudioOutput(&output);
 
-    QSignalSpy metadataChangedSpy(&player, SIGNAL(metaDataChanged()));
+    QSignalSpy metadataChangedSpy(&player, &QMediaPlayer::metaDataChanged);
 
     player.setSource(QUrl::fromLocalFile(fileName));
     player.play();
 
-    QTRY_VERIFY(metadataChangedSpy.size() > 0);
+    int metadataChangedRequiredCount = isGStreamerPlatform() ? 2 : 1;
 
-    QCOMPARE(player.metaData().value(QMediaMetaData::Title).toString(), metaData.value(QMediaMetaData::Title).toString());
+    QTRY_VERIFY(metadataChangedSpy.size() >= metadataChangedRequiredCount);
+
+    QCOMPARE(player.metaData().value(QMediaMetaData::Title).toString(),
+             metaData.value(QMediaMetaData::Title).toString());
     auto lang = player.metaData().value(QMediaMetaData::Language).value<QLocale::Language>();
     if (lang != QLocale::AnyLanguage)
         QCOMPARE(lang, metaData.value(QMediaMetaData::Language).value<QLocale::Language>());
     QCOMPARE(player.metaData().value(QMediaMetaData::Description).toString(), metaData.value(QMediaMetaData::Description).toString());
+    QVERIFY(player.metaData().value(QMediaMetaData::Resolution).isValid());
 
-    metadataChangedSpy.clear();
+    if (isGStreamerPlatform())
+        QVERIFY(player.metaData().value(QMediaMetaData::Date).isValid());
 
     player.stop();
     player.setSource({});
