@@ -1,7 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
-
-//TESTED_COMPONENT=src/multimedia
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtTest/QtTest>
 
@@ -13,7 +11,7 @@
 #include <private/qplatformmediaplayer_p.h>
 #include <qobject.h>
 
-#include "qmockintegration_p.h"
+#include "qmockintegration.h"
 #include "qmockmediaplayer.h"
 #include "qmockaudiooutput.h"
 #include "qvideosink.h"
@@ -21,6 +19,8 @@
 
 QT_USE_NAMESPACE
 
+Q_ENABLE_MOCK_MULTIMEDIA_PLUGIN
+
 class AutoConnection
 {
 public:
@@ -53,6 +53,9 @@ public slots:
     void cleanup();
 
 private slots:
+    void play_doesNotResetError();
+    void setSource_resetsError();
+    void setSourceDevice_resetsError();
     void testValid();
     void testMedia_data();
     void testMedia();
@@ -98,7 +101,6 @@ private slots:
 private:
     void setupCommonTestData();
 
-    QMockIntegration *mockIntegration;
     QMockMediaPlayer *mockPlayer;
     QAudioOutput *audioOutput = nullptr;
     QMediaPlayer *player;
@@ -175,7 +177,7 @@ void tst_QMediaPlayer::setupCommonTestData()
                                 qreal(1)  << QMediaPlayer::NoError << QString();
     QTest::newRow("valid+error") << true << QMediaPlayer::StoppedState << QMediaPlayer::InvalidMedia <<
                                 QUrl(QUrl("http://example.com/stream")) << qint64(0) << qint64(0) << false << 50 << false << false << 0 <<
-                                qreal(0) << QMediaPlayer::ResourceError << QString("Resource unavailable");
+                                qreal(0) << QMediaPlayer::ResourceError << QStringLiteral("Resource unavailable");
 }
 
 void tst_QMediaPlayer::initTestCase()
@@ -188,19 +190,58 @@ void tst_QMediaPlayer::cleanupTestCase()
 
 void tst_QMediaPlayer::init()
 {
-    mockIntegration = new QMockIntegration;
     player = new QMediaPlayer;
-    mockPlayer = mockIntegration->lastPlayer();
-    Q_ASSERT(mockPlayer);
+    mockPlayer = QMockIntegration::instance()->lastPlayer();
+    QTEST_ASSERT(mockPlayer);
     audioOutput = new QAudioOutput;
     player->setAudioOutput(audioOutput);
-    Q_ASSERT(mockPlayer->m_audioOutput != nullptr);
+    QTEST_ASSERT(mockPlayer->m_audioOutput != nullptr);
 }
 
 void tst_QMediaPlayer::cleanup()
 {
     delete player;
-    delete mockIntegration;
+}
+
+void tst_QMediaPlayer::play_doesNotResetError()
+{
+    player->setSource({ "https://dummy/dummy.mp3" });
+
+    mockPlayer->setError(QMediaPlayer::AccessDeniedError);
+    mockPlayer->setErrorString(QStringLiteral(u"Error"));
+    QSignalSpy errorSpy{ player, &QMediaPlayer::errorChanged };
+
+    player->play();
+
+    QCOMPARE_EQ(player->error(), QMediaPlayer::AccessDeniedError);
+    QCOMPARE_EQ(player->errorString(), QStringLiteral(u"Error"));
+    QCOMPARE_EQ(errorSpy.size(), 0);
+}
+
+void tst_QMediaPlayer::setSource_resetsError()
+{
+    mockPlayer->setError(QMediaPlayer::AccessDeniedError);
+    mockPlayer->setErrorString(QStringLiteral(u"Error"));
+    QSignalSpy errorSpy{ player, &QMediaPlayer::errorChanged };
+
+    player->setSource({ "https://dummy/dummy.mp3" });
+
+    QCOMPARE_EQ(player->error(), QMediaPlayer::NoError);
+    QVERIFY(player->errorString().isEmpty());
+    QCOMPARE_EQ(errorSpy.size(), 1);
+}
+
+void tst_QMediaPlayer::setSourceDevice_resetsError()
+{
+    mockPlayer->setError(QMediaPlayer::AccessDeniedError);
+    mockPlayer->setErrorString(QStringLiteral(u"Error"));
+    QSignalSpy errorSpy{ player, &QMediaPlayer::errorChanged };
+
+    player->setSourceDevice(nullptr, { "https://dummy/dummy.mp3" });
+
+    QCOMPARE_EQ(player->error(), QMediaPlayer::NoError);
+    QVERIFY(player->errorString().isEmpty());
+    QCOMPARE_EQ(errorSpy.size(), 1);
 }
 
 void tst_QMediaPlayer::testValid()
@@ -297,32 +338,40 @@ void tst_QMediaPlayer::testPosition()
     QVERIFY(player->duration() == duration);
 
     if (seekable) {
-        { QSignalSpy spy(player, SIGNAL(positionChanged(qint64)));
-        player->setPosition(position);
-        QCOMPARE(player->position(), position);
-        QCOMPARE(spy.size(), 0); }
+        {
+            QSignalSpy spy(player, &QMediaPlayer::positionChanged);
+            player->setPosition(position);
+            QCOMPARE(player->position(), position);
+            QCOMPARE(spy.size(), 0);
+        }
 
         mockPlayer->setPosition(position);
-        { QSignalSpy spy(player, SIGNAL(positionChanged(qint64)));
-        player->setPosition(0);
-        QCOMPARE(player->position(), qint64(0));
-        QCOMPARE(spy.size(), position == 0 ? 0 : 1); }
+        {
+            QSignalSpy spy(player, &QMediaPlayer::positionChanged);
+            player->setPosition(0);
+            QCOMPARE(player->position(), qint64(0));
+            QCOMPARE(spy.size(), position == 0 ? 0 : 1);
+        }
 
         mockPlayer->setPosition(position);
-        { QSignalSpy spy(player, SIGNAL(positionChanged(qint64)));
-        player->setPosition(duration);
-        QCOMPARE(player->position(), duration);
-        QCOMPARE(spy.size(), position == duration ? 0 : 1); }
+        {
+            QSignalSpy spy(player, &QMediaPlayer::positionChanged);
+            player->setPosition(duration);
+            QCOMPARE(player->position(), duration);
+            QCOMPARE(spy.size(), position == duration ? 0 : 1);
+        }
 
         mockPlayer->setPosition(position);
-        { QSignalSpy spy(player, SIGNAL(positionChanged(qint64)));
-        player->setPosition(-1);
-        QCOMPARE(player->position(), qint64(0));
-        QCOMPARE(spy.size(), position == 0 ? 0 : 1); }
+        {
+            QSignalSpy spy(player, &QMediaPlayer::positionChanged);
+            player->setPosition(-1);
+            QCOMPARE(player->position(), qint64(0));
+            QCOMPARE(spy.size(), position == 0 ? 0 : 1);
+        }
 
     }
     else {
-        QSignalSpy spy(player, SIGNAL(positionChanged(qint64)));
+        QSignalSpy spy(player, &QMediaPlayer::positionChanged);
         player->setPosition(position);
 
         QCOMPARE(player->position(), position);
@@ -345,25 +394,33 @@ void tst_QMediaPlayer::testVolume()
     QVERIFY(audioOutput->volume() == vol);
 
     if (valid) {
-        { QSignalSpy spy(audioOutput, SIGNAL(volumeChanged(float)));
-        audioOutput->setVolume(.1f);
-        QCOMPARE(audioOutput->volume(), .1f);
-        QCOMPARE(spy.size(), 1); }
+        {
+            QSignalSpy spy(audioOutput, &QAudioOutput::volumeChanged);
+            audioOutput->setVolume(.1f);
+            QCOMPARE(audioOutput->volume(), .1f);
+            QCOMPARE(spy.size(), 1);
+        }
 
-        { QSignalSpy spy(audioOutput, SIGNAL(volumeChanged(float)));
-        audioOutput->setVolume(-1000.f);
-        QCOMPARE(audioOutput->volume(), 0.f);
-        QCOMPARE(spy.size(), 1); }
+        {
+            QSignalSpy spy(audioOutput, &QAudioOutput::volumeChanged);
+            audioOutput->setVolume(-1000.f);
+            QCOMPARE(audioOutput->volume(), 0.f);
+            QCOMPARE(spy.size(), 1);
+        }
 
-        { QSignalSpy spy(audioOutput, SIGNAL(volumeChanged(float)));
-        audioOutput->setVolume(1.f);
-        QCOMPARE(audioOutput->volume(), 1.f);
-        QCOMPARE(spy.size(), 1); }
+        {
+            QSignalSpy spy(audioOutput, &QAudioOutput::volumeChanged);
+            audioOutput->setVolume(1.f);
+            QCOMPARE(audioOutput->volume(), 1.f);
+            QCOMPARE(spy.size(), 1);
+        }
 
-        { QSignalSpy spy(audioOutput, SIGNAL(volumeChanged(float)));
-        audioOutput->setVolume(1000.f);
-        QCOMPARE(audioOutput->volume(), 1.f);
-        QCOMPARE(spy.size(), 0); }
+        {
+            QSignalSpy spy(audioOutput, &QAudioOutput::volumeChanged);
+            audioOutput->setVolume(1000.f);
+            QCOMPARE(audioOutput->volume(), 1.f);
+            QCOMPARE(spy.size(), 0);
+        }
     }
 }
 
@@ -384,7 +441,7 @@ void tst_QMediaPlayer::testMuted()
         audioOutput->setVolume(vol);
         QVERIFY(audioOutput->isMuted() == muted);
 
-        QSignalSpy spy(audioOutput, SIGNAL(mutedChanged(bool)));
+        QSignalSpy spy(audioOutput, &QAudioOutput::mutedChanged);
         audioOutput->setMuted(!muted);
         QCOMPARE(audioOutput->isMuted(), !muted);
         QCOMPARE(audioOutput->volume(), vol);
@@ -445,7 +502,7 @@ void tst_QMediaPlayer::testPlaybackRate()
         mockPlayer->setPlaybackRate(playbackRate);
         QVERIFY(player->playbackRate() == playbackRate);
 
-        QSignalSpy spy(player, SIGNAL(playbackRateChanged(qreal)));
+        QSignalSpy spy(player, &QMediaPlayer::playbackRateChanged);
         player->setPlaybackRate(playbackRate + 0.5f);
         QCOMPARE(player->playbackRate(), playbackRate + 0.5f);
         QCOMPARE(spy.size(), 1);
@@ -512,19 +569,25 @@ void tst_QMediaPlayer::testPlay()
     player->setSource(mediaContent);
     mockPlayer->setState(state);
     QCOMPARE(player->playbackState(), state);
+    QCOMPARE(player->isPlaying(), state == QMediaPlayer::PlayingState);
     QCOMPARE(player->source(), mediaContent);
 
-    QSignalSpy spy(player, SIGNAL(playbackStateChanged(QMediaPlayer::PlaybackState)));
+    QSignalSpy spy(player, &QMediaPlayer::playbackStateChanged);
+    QSignalSpy playingChanged(player, &QMediaPlayer::playingChanged);
 
     player->play();
 
     if (!valid || mediaContent.isEmpty())  {
         QCOMPARE(player->playbackState(), QMediaPlayer::StoppedState);
+        QVERIFY(!player->isPlaying());
         QCOMPARE(spy.size(), 0);
+        QVERIFY(playingChanged.empty());
     }
     else {
         QCOMPARE(player->playbackState(), QMediaPlayer::PlayingState);
+        QVERIFY(player->isPlaying());
         QCOMPARE(spy.size(), state == QMediaPlayer::PlayingState ? 0 : 1);
+        QCOMPARE_EQ(playingChanged.size(), state == QMediaPlayer::PlayingState ? 0 : 1);
     }
 }
 
@@ -543,19 +606,25 @@ void tst_QMediaPlayer::testPause()
     player->setSource(mediaContent);
     mockPlayer->setState(state);
     QVERIFY(player->playbackState() == state);
+    QCOMPARE(player->isPlaying(), state == QMediaPlayer::PlayingState);
     QVERIFY(player->source() == mediaContent);
 
-    QSignalSpy spy(player, SIGNAL(playbackStateChanged(QMediaPlayer::PlaybackState)));
+    QSignalSpy spy(player, &QMediaPlayer::playbackStateChanged);
+    QSignalSpy playingChanged(player, &QMediaPlayer::playingChanged);
 
     player->pause();
 
+    QVERIFY(!player->isPlaying());
+
     if (!valid || mediaContent.isEmpty()) {
         QCOMPARE(player->playbackState(), QMediaPlayer::StoppedState);
         QCOMPARE(spy.size(), 0);
+        QCOMPARE(playingChanged.size(), 0);
     }
     else {
         QCOMPARE(player->playbackState(), QMediaPlayer::PausedState);
         QCOMPARE(spy.size(), state == QMediaPlayer::PausedState ? 0 : 1);
+        QCOMPARE(playingChanged.size(), state == QMediaPlayer::PlayingState ? 1 : 0);
     }
 }
 
@@ -572,19 +641,25 @@ void tst_QMediaPlayer::testStop()
     player->setSource(mediaContent);
     mockPlayer->setState(state);
     QVERIFY(player->playbackState() == state);
+    QCOMPARE(player->isPlaying(), state == QMediaPlayer::PlayingState);
     QVERIFY(player->source() == mediaContent);
 
-    QSignalSpy spy(player, SIGNAL(playbackStateChanged(QMediaPlayer::PlaybackState)));
+    QSignalSpy spy(player, &QMediaPlayer::playbackStateChanged);
+    QSignalSpy playingChanged(player, &QMediaPlayer::playingChanged);
 
     player->stop();
 
+    QVERIFY(!player->isPlaying());
+
     if (mediaContent.isEmpty() || state == QMediaPlayer::StoppedState) {
         QCOMPARE(player->playbackState(), QMediaPlayer::StoppedState);
         QCOMPARE(spy.size(), 0);
+        QCOMPARE(playingChanged.size(), 0);
     }
     else {
         QCOMPARE(player->playbackState(), QMediaPlayer::StoppedState);
         QCOMPARE(spy.size(), 1);
+        QCOMPARE(playingChanged.size(), state == QMediaPlayer::PlayingState ? 1 : 0);
     }
 }
 
@@ -601,8 +676,8 @@ void tst_QMediaPlayer::testMediaStatus()
     mockPlayer->setMediaStatus(QMediaPlayer::NoMedia);
     mockPlayer->setBufferStatus(bufferProgress);
 
-    QSignalSpy statusSpy(player, SIGNAL(mediaStatusChanged(QMediaPlayer::MediaStatus)));
-    QSignalSpy bufferSpy(player, SIGNAL(bufferProgressChanged(float)));
+    QSignalSpy statusSpy(player, &QMediaPlayer::mediaStatusChanged);
+    QSignalSpy bufferSpy(player, &QMediaPlayer::bufferProgressChanged);
 
     QCOMPARE(player->mediaStatus(), QMediaPlayer::NoMedia);
 
@@ -771,9 +846,9 @@ void tst_QMediaPlayer::testQrc()
     mockPlayer->setState(QMediaPlayer::PlayingState, QMediaPlayer::NoMedia);
     mockPlayer->setStreamPlaybackSupported(backendHasStream);
 
-    QSignalSpy mediaSpy(player, SIGNAL(sourceChanged(QUrl)));
-    QSignalSpy statusSpy(player, SIGNAL(mediaStatusChanged(QMediaPlayer::MediaStatus)));
-    QSignalSpy errorSpy(player, SIGNAL(errorOccurred(QMediaPlayer::Error,const QString&)));
+    QSignalSpy mediaSpy(player, &QMediaPlayer::sourceChanged);
+    QSignalSpy statusSpy(player, &QMediaPlayer::mediaStatusChanged);
+    QSignalSpy errorSpy(player, &QMediaPlayer::errorOccurred);
 
     player->setSource(mediaContent);
 
